import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { Controller } from '@nestjs/common';
import { CommandBus, QueryBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { DatabaseException } from 'src/modules/database/src/exceptions/DatabaseException';
import { CreateAuthCommand } from '../../commands/create-auth.command';
import { UpdateAuthCommand } from '../../commands/update-auth.command';
import { CreateAuthRequest } from '../../domain/dtos/create-auth.request';
import { UpdateAuthRequest } from '../../domain/dtos/update-auth.request';
import { ValidateAuthRequest } from '../../domain/dtos/validate-auth.request';
import { Auth } from '../../domain/entities/auth';
import { ValidateAuthQuery } from '../../queries/validate-auth.query';
import { AuthPresenter } from './auth.presenter';

@Controller()
export class AuthController {
  constructor(
    private readonly _commandBus: CommandBus,
    private readonly _queryBus: QueryBus,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  @GrpcMethod('AuthService', 'Validate')
  async validate(data: ValidateAuthRequest): Promise<AuthPresenter> {
    try {
      const auth = await this._queryBus.execute(
        new ValidateAuthQuery(data.username, data.password),
      );
      return this._mapper.map(auth, Auth, AuthPresenter);
    } catch (e) {
      throw new RpcException({
        code: 7,
        message: 'Permission denied',
      });
    }
  }

  @GrpcMethod('AuthService', 'Create')
  async createUser(data: CreateAuthRequest): Promise<AuthPresenter> {
    try {
      const auth = await this._commandBus.execute(new CreateAuthCommand(data));
      return this._mapper.map(auth, Auth, AuthPresenter);
    } catch (e) {
      if (e instanceof DatabaseException) {
        if (e.message.includes('Already exists')) {
          throw new RpcException({
            code: 6,
            message: 'Auth already exists',
          });
        }
      }
      throw new RpcException({});
    }
  }

  @GrpcMethod('AuthService', 'Update')
  async updateAuth(data: UpdateAuthRequest): Promise<AuthPresenter> {
    try {
      const auth = await this._commandBus.execute(new UpdateAuthCommand(data));

      return this._mapper.map(auth, Auth, AuthPresenter);
    } catch (e) {
      throw new RpcException({
        code: 7,
        message: 'Permission denied',
      });
    }
  }
}
