import { classes } from '@automapper/classes';
import { AutomapperModule } from '@automapper/nestjs';
import { Test, TestingModule } from '@nestjs/testing';
import { UpdateAuthCommand } from '../../commands/update-auth.command';
import { UpdateAuthRequest } from '../../domain/dtos/update-auth.request';
import { Auth } from '../../domain/entities/auth';
import * as bcrypt from 'bcrypt';
import { UpdateAuthUseCase } from '../../domain/usecases/update-auth.usecase';
import { AuthRepository } from '../../adapters/secondaries/auth.repository';
import { AuthProfile } from '../../mappers/auth.profile';

const originalAuth: Auth = new Auth();
originalAuth.uuid = 'bb281075-1b98-4456-89d6-c643d3044a91';
originalAuth.username = 'john.doe@email.com';
originalAuth.password = 'encrypted_password';

const updateUsernameAuthRequest: UpdateAuthRequest = {
  uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
  username: 'johnny.doe@email.com',
};

const updatePasswordAuthRequest: UpdateAuthRequest = {
  uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
  password: 'John1234',
};

const updateAuthRequest: UpdateAuthRequest = {
  uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
  username: 'johnny.doe@email.com',
  password: 'John1234',
};

const updateUsernameAuthCommand: UpdateAuthCommand = new UpdateAuthCommand(
  updateUsernameAuthRequest,
);

const updatePasswordAuthCommand: UpdateAuthCommand = new UpdateAuthCommand(
  updatePasswordAuthRequest,
);

const updateAuthCommand: UpdateAuthCommand = new UpdateAuthCommand(
  updateAuthRequest,
);

const mockAuthRepository = {
  update: jest.fn().mockImplementation((uuid: string, params: any) => {
    if (params.username && params.password) {
      const auth: Auth = { ...originalAuth };
      auth.username = params.username;
      auth.password = params.password;
      return Promise.resolve(auth);
    }
    if (params.username) {
      const auth: Auth = { ...originalAuth };
      auth.username = params.username;
      return Promise.resolve(auth);
    }
    if (params.password) {
      const auth: Auth = { ...originalAuth };
      auth.password = params.password;
      return Promise.resolve(auth);
    }
  }),
};

describe('UpdateAuthUseCase', () => {
  let updateAuthUseCase: UpdateAuthUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AutomapperModule.forRoot({ strategyInitializer: classes() })],

      providers: [
        {
          provide: AuthRepository,
          useValue: mockAuthRepository,
        },
        UpdateAuthUseCase,
        AuthProfile,
      ],
    }).compile();

    updateAuthUseCase = module.get<UpdateAuthUseCase>(UpdateAuthUseCase);
  });

  it('should be defined', () => {
    expect(updateAuthUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should update the username of an Auth', async () => {
      const updatedAuth: Auth = await updateAuthUseCase.execute(
        updateUsernameAuthCommand,
      );

      expect(updatedAuth.username).toBe(updateUsernameAuthRequest.username);
    });
    it('should update the password of an Auth', async () => {
      const updatedAuth: Auth = await updateAuthUseCase.execute(
        updatePasswordAuthCommand,
      );
      expect(
        bcrypt.compareSync(
          updatePasswordAuthRequest.password,
          updatedAuth.password,
        ),
      ).toBeTruthy();
    });
    it('should update the username and the password of an Auth', async () => {
      const updatedAuth: Auth = await updateAuthUseCase.execute(
        updateAuthCommand,
      );
      expect(updatedAuth.username).toBe(updateAuthRequest.username);
      expect(
        bcrypt.compareSync(updateAuthRequest.password, updatedAuth.password),
      ).toBeTruthy();
    });
  });
});
